
// Auteur Martin Dufour, St-Nicolas, Québec, Canada 
// EMAIL: martindufour@live.ca
// Mise à jour: Novembre 2017
// Version 1.0
//
// Ce programme permet d'actionner un moteur de porte de garage utilisé pour déplacer un toit d'observatoire à toit ouvrant (roll off). 
// Il fonctionne grâce a un relais 5volts qui simule l'action de presser le bouton pressoir d'un interrupteur mural qui commande l'ouverture
// et/ou la fermeture d'un moteur de  porte de garage et grace à deux contacts magnétiques, il permet en tout temps de connaitre la position
// ou l'état du toit (OUVERT ou FERMÉ). Il est primordial de connaitre le statut du toit (OUVERT ou FERMÉ), surtout dans un contexte d'automatisation
// et de contrôle à distance de votre observatoire via un réseau local ou par Internet.
// Les codes font référence à une porte de garage (DOOR). Il faut comprendre que le moteur de porte de garage est utilisé pour ouvrir et fermer
// un toit d'observatoire à toit ouvrant (roll off. Donc pensez que lorsqu'on fait référence à une porte "DOOR", il s'agit en fait du TOIT amovible
// de l'observatoire.


// Variables

int doorPin  = 10;
int doorPinClose  = 12;
int relayPin = 7;
int inByte = 0;       
int doorState = 0; 
int doorStateClose = 0; 

// Le protocole de communication avec le driver ASCOM est assez simple et repose sur 5 commandes de base que voici:

int OD = 79;          // Open door, le driver ASCOM va transmettre un O (OPEN)soit le code ascii 79
int CD = 67;          // Close door, le driver ASCOM va transmettre un C (CLOSE) soit le code ascii 67
int DS = 83;          // Door status, le driver ASCOM va transmettre un S (STATUS) soit le code ascii 83
int INFOVERSION = 63; // Info version du code de la carte Arduino, le driver ASCOM va transmettre un ? soit le code ascii 63
int BYPASS = 33;      // Permet de bypasser la vérification des capteurs et de faire déplacer le toit, le driver ASCOM va transmetrte un ! soit le code ascii 33

void setup() {
  digitalWrite(relayPin, HIGH);
  pinMode(doorPin, INPUT);
  pinMode(doorPinClose, INPUT);
  pinMode(relayPin, OUTPUT);
  Serial.begin(9600); 
}

// La loop attend une commande provenant du driver ASCOM

void loop() 
{
  if (Serial.available() > 0) 
  {
    inByte = Serial.read(); // Lecture de la commande du port série, la commande va provenir du driver ASCOM

    if (inByte == DS) // Si le driver ASCOM transmet un S pour STATUT, alors on va retourner le statut du toit (O pour OPEN ou C pour CLOSE)
    {
      GetDoorState();
    }
      
    if (inByte == OD ) // Si le driver ASCOM transmet un O pour OPEN DOOR, alors on va ouvrir le toit
    {
      OpenDoor();
    }

    if (inByte == CD ) // Si le driver ASCOM transmet un C pour CLOSE DOOR, alors on va fermer le toit
    {
      CloseDoor();
    }

    
    if (inByte == INFOVERSION ) // Si le driver ASCOM transmet un ? pour INFO VERSION, alors on va retourner la version du code de la carte Arduino
    {
      InfoVersion();
    }

    
    if (inByte == BYPASS ) // Si le driver ASCOM transmet un ! pour MOVE DOOR, alors on va ouvrir ou fermer le toit dans la vérification sur la position la porte
    {
      moveDoor();
    }
  }      
  
  delay(100);
  
}



// Cette fonction permet de fermer le toit, elle va cependant s'assurer que le toit est bien en position OUVERT avant.   

void CloseDoor()
{
  doorStateClose = digitalRead(doorPinClose); // Interrogation du sensor (contact magnétique du toit en position CLOSE) attribué à l'état CLOSE
  doorState = digitalRead(doorPin);           // Interrogation du sensor (contact magnétique du toit en position OPEN)attribué à l'état OPEN
  if (! (HIGH == doorStateClose && HIGH == doorState ) )  // On s'assure que le toit est bien en position OPEN
  {
    if (HIGH == doorStateClose) 
    {
      moveDoor();
    }
  }
}


// Cette fonction permet d'ouvrir le toit, elle va cependant s'assurer que le toit est bien en position CLOSE avant.   

void OpenDoor()
{
  doorStateClose = digitalRead(doorPinClose); // Interrogation du sensor (contact magnétique toit FERMÉ)attribué à l'état CLOSE
  doorState = digitalRead(doorPin);           // Interrogation du sensor (contact magnétique toit OUVERT)attribué à l'état OPEN
  if (! (HIGH == doorStateClose && HIGH == doorState ) ) // On s'assure que le toit est bien en position (fermé)CLOSE
  {
    if (HIGH == doorState) 
    {
       moveDoor();
    }
  }
}


// Cette fonction retourne le statut du toit: OPEN, CLOSE ou en MOVE
// Le statut est retourné au driver ASCOM via un port série

void GetDoorState()
{
  doorState = digitalRead(doorPin);
  if (LOW == doorState) 
  {
     Serial.print("O");
  }
  doorStateClose = digitalRead(doorPinClose);
  if (LOW == doorStateClose) 
  {
     Serial.print("C");
  }

  if (HIGH == doorStateClose && HIGH == doorState ) 
  {
     Serial.print("M");
  }
 
  
  delay(200);
}


// Cette fonction retourne la version du code Arduino
void InfoVersion()
{
     Serial.print("1.0");
}


// Cette fonction actionne le relais afin de faire déplacer le toit
void moveDoor()
{
  digitalWrite(relayPin, LOW);
  delay(500);
  digitalWrite(relayPin, HIGH);
}

